% Programm Gauss-Newton (Kap. 7)
% nichtlineares Ausgleichsproblem
% f(x) = b*sin(x -  a) 
% 
% x_k   y_k
% -1,5  -0,4 
% 0     0,1
% 1     0,45
% 2     0,35
% 3     0
% 4     -0,4 
% 4,5   -0,5 
%
% r(a,b,c) = ( b*sin(a*x1 + c)  - y1;
%            b*sin(a*x2 + c)  - y2;
%            ...
%            b*sin(a*xn + c)  - yn )
% r'(a,b,c) = ...
%
% Aufruf: [a,b,c,it] = gaussnewton3(a0,b0,c0,alpha);
%
function [a,b,c,it] = gaussnewton3(a0,b0,c0,alpha);
xx = [ -1.5 0 1 2 3 4 4.5];
yy = [ -0.4 0.1 0.45 0.35 0 -0.4 -0.5];
a = a0; b = b0; c = c0;
M = Rs(a,b,c,xx,yy);
abc = [a; b; c];
ys = -R(a,b,c,xx,yy);
s = 10;
epsilon = 0.000001;
it = 0;
while (norm(s) > epsilon && it < 40)
   it = it + 1;
   s = M\ys;
   as = s + abc;
   abc = alpha*as + (1 - alpha)*abc;
   M = Rs(abc(1),abc(2),abc(3),xx,yy);
   ys = -R(abc(1),abc(2),abc(3),xx,yy);
   0.5*norm(R(abc(1),abc(2),abc(3),xx,yy))^2
end 
s
a = abc(1);
b = abc(2);
c = abc(3);
x = linspace(-2,5,20);
for j=1:20
   y(j) = b*sin(a*x(j) + c);
end
%
plot(x,y,'b-');
xlabel('x');
ylabel('y=f(x)');
hold on
plot(xx,yy,'*');
hold off
end % function
%
function rabc = R(a,b,c,xx,yy);
r = zeros(7,1);
for j=1:7
   r(j) = b*sin(a*xx(j) + c) - yy(j);
end
rabc = r;
end % function
%
function rsabc = Rs(a,b,c,xx,yy);
r = zeros(7,3);
for j=1:7
   r(j,1) = b*cos(a*xx(j)+c)*xx(j); 
   r(j,2) = sin(a*xx(j)+c); 
   r(j,3) = b*cos(a*xx(j) + c);
end
rsabc = r;
end % function
